/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "mozCellML.h"
#include "mozCellMLProtocol.h"
#include "mozCellMLChannel.h"
#include <nsString.h>
#include <nsArray.h>
#include <necko/nsNetUtil.h>

NS_IMPL_ISUPPORTS1(mozCellMLProtocol,
                   nsIProtocolHandler
                  )


mozCellMLProtocol::mozCellMLProtocol()
{
  LEAKTRACE_CREATED(mozCellMLProtocol)
}

NS_IMETHODIMP
mozCellMLProtocol::GetScheme(nsACString& scheme)
{
  scheme.Assign(NS_LITERAL_CSTRING("mozcellml"));
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLProtocol::GetDefaultPort(PRInt32* port)
{
  *port = -1;
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLProtocol::GetProtocolFlags(PRUint32* flags)
{
  *flags = nsIProtocolHandler::URI_NORELATIVE | nsIProtocolHandler::URI_NOAUTH;
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLProtocol::NewURI(const nsACString& spec,
                          const char* charset,
                          nsIURI* base,
                          nsIURI** out)
{
  nsCAutoString url;

  PRInt32 colon = spec.FindChar(':');
  nsCAutoString sspec(Substring(spec, colon + 1));

  nsresult ret;
  nsCOMPtr<nsIURI> s = do_CreateInstance(NS_SIMPLEURI_CONTRACTID, &ret);
  *out = s;
  NS_ADDREF(*out);
  if (ret != NS_OK)
    return ret;
  (*out)->SetSpec(NS_LITERAL_CSTRING("mozcellml:") + sspec);
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLProtocol::NewChannel(nsIURI* uri, nsIChannel** chan)
{
  *chan = NULL;
  nsCAutoString url;
  nsresult rv;

  rv = uri->GetSpec(url);
  NS_ENSURE_SUCCESS(rv, rv);

  PRInt32 colon = url.FindChar(':');
  url = Substring(url, colon + 1);
  nsCOMPtr<nsIURI> underURI = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
  NS_ENSURE_SUCCESS(rv, rv);
#ifdef DEBUG_MODE
  printf("Made a standardurl...\n");
#endif

  rv = underURI->SetSpec(url);
  NS_ENSURE_SUCCESS(rv, rv);
#ifdef DEBUG_MODE
  printf("Set the underURI...\n");
#endif

  nsCOMPtr<nsIChannel> underChannel;
  nsCOMPtr<nsIIOService> pService(do_GetIOService(&rv));
  rv = pService->NewChannel(url, nsnull, nsnull, getter_AddRefs(underChannel));
  NS_ENSURE_SUCCESS(rv, rv);
#ifdef DEBUG_MODE
  printf("Made a NewChannel...\n");
#endif
  underChannel->SetOriginalURI(underURI);
  NS_ENSURE_SUCCESS(rv, rv);
#ifdef DEBUG_MODE
  printf("Set the original URL...\n");
#endif

  mozCellMLChannel* mchan = new mozCellMLChannel(underChannel);
  NS_ADDREF(mchan);
  CallQueryInterface(mchan, chan);
  NS_RELEASE(mchan);

  (*chan)->SetOriginalURI(uri);

#ifdef DEBUG_MODE
  printf("Everything done.\n");
#endif
  return NS_OK;
}

NS_IMETHODIMP
mozCellMLProtocol::AllowPort(PRInt32 port,
                             const char* scheme, PRBool* override)
{
  *override = PR_FALSE;
  return NS_OK;
}
