function InsertVariable(box, name, type, initialValue, units)
{
  var li = document.createElement("listitem");
  li.setAttribute("id", name);
  var lc = document.createElement("listcell");
  lc.setAttribute("label", name);
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("label", type);
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("label", initialValue);
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("label", units);
  li.appendChild(lc);
  box.appendChild(li);
}

function ChangeInitialValue()
{
  var tf = document.getElementById("fieldvalue");
  var value = parseFloat(tf.value);
  if (isNaN(value))
    return;
  var tree = document.getElementById('variable-tree');
  var idx = tree.selectedIndex;
  if (idx == -1)
    return;
  if ((window.variables[idx].type ==
       Components.interfaces.mozCellMLIVariable.BOUND) ||
       (window.variables[idx].type ==
        Components.interfaces.mozCellMLIVariable.SIMPLE))
    return;

  window.variables[idx].initialValue = value;

  // Because of the CellML 1.1 initial_value="name" feature, we need
  // to update the initial value display of all variables in the list...
  var treechildren = tree.childNodes;
  var ntc = treechildren.length;
  var j = 0;
  for (var i = 0; i < ntc; i++)
  {
    var tc = treechildren.item(i);
    if (tc.localName != "listitem")
      continue;
    try
    {
      tc.childNodes.item(2).setAttribute("label", window.variables[j].initialValue);
    }
    catch (e) {}
    j++;
  }

  return value;
}

function SelectVariable()
{
  var tree = document.getElementById('variable-tree');
  var idx = tree.selectedIndex;
  if (idx == -1)
    return;
  var listitem = tree.selectedItem;
  var valueSubs = listitem.childNodes;
  var value = valueSubs.item(2).getAttribute("label");
  var name = valueSubs.item(0).getAttribute("label");
  var tf = document.getElementById("fieldvalue");
  tf.readonly = (window.variables[idx].type ==
                 Components.interfaces.mozCellMLIVariable.BOUND) ||
                (window.variables[idx].type ==
                 Components.interfaces.mozCellMLIVariable.SIMPLE);
  tf.value = value;
  var lb = document.getElementById("fieldname");
  lb.value = name + ": ";
}

function TypeToString(v)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  switch (v)
  {
    case Components.interfaces.mozCellMLIVariable.CONSTANT:
      return strb.getString("varTypeConstant");
    case Components.interfaces.mozCellMLIVariable.SIMPLE:
      return strb.getString("varTypeMathematical");
    case Components.interfaces.mozCellMLIVariable.DIFFERENTIAL:
      return strb.getString("varTypeDifferential");
    case Components.interfaces.mozCellMLIVariable.BOUND:
      return strb.getString("varTypeBound");
  }
}

function AddVariables(box)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  window.model =
    window.arguments[0].QueryInterface(Components.interfaces.mozICellML);

  if (window.arguments[1] == null)
  {
    window.model.setStringBundle(strb.stringBundle);
    window.model.initialise();
  }

  var statusBar = document.getElementById('status-bar-display');
  var validity = window.model.checkValidity();
  if (validity == "")
    statusBar.label = strb.getString("modelLoadNoError");
  else
    statusBar.label = strb.getFormattedString("modelInvalidReason", [validity]);

  if (window.arguments[1] != true)
    window.integrator = window.arguments[1];
  var varit = window.model.getAllVariables(Components.interfaces.
                                           mozICellML.NO_INTERFACE_IN);
  window.variables = new Array();
  var i = 0;
  while (varit.hasMoreElements())
  {
    var v = varit.getNext();
    v = v.QueryInterface(Components.interfaces.mozCellMLIVariable);
    if (v.type == Components.interfaces.mozCellMLIVariable.BOUND)
    {
      window.bound = v.name;
      window.boundunit = v.units;
      var boundname = document.getElementById("boundname");
      boundname.value = window.bound + " interval: (";
      var unitsep = document.getElementById("unitsep");
      unitsep.value = window.boundunit + " ,";
      var unitstop = document.getElementById("unitstop");
      unitstop.value = window.boundunit + ")";
      var unitgrid = document.getElementById("unitgrid");
      unitgrid.value = window.boundunit;
    }
    window.variables[i] = v;
    var iv = "";
    try
    {
      iv = v.initialValue;
    } catch(e)
    {
      if (e.result != Components.results.NS_ERROR_NOT_AVAILABLE)
      {
        var validity = window.model.checkValidity();
        statusBar.label =
          strb.getFormattedString("modelInvalidReason", [validity]);
      }
    }
    InsertVariable(box, v.name, TypeToString(v.type), iv, v.units);
    i++;
  }
  window.nvariables = i;

  if (window.integrator != null)
  {
    // XXX we need to synchronise whether an integration is in
    // progress between all the windows that share integrators.
    ChangeIntegratorButtons(document, true, false);
  }
}

function SelectLocalSave(returnto)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseSaveFilename"),
            nsIFilePicker.modeSave);
    fp.appendFilter(strb.getString("allCellML"), "*.cml; *.xml");
    fp.appendFilter("CML", "*.cml");
    fp.appendFilter("XML", "*.xml");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() == nsIFilePicker.returnCancel)
      return;
    returnto.value = fp.file.path;
  }
  catch (ex)
  {
  }
}

function SelectLocalExport(returnto)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseCSVExportFilename"),
            nsIFilePicker.modeSave);
    fp.appendFilter("CSV", "*.csv");
    fp.appendFilter(strb.getString("textFiles"), "*.txt");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() == nsIFilePicker.returnCancel)
      return;
    returnto.value = fp.file.path;
  }
  catch (ex)
  {
  }
}

function UserRequestSave(saveto, model)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var file = Components.classes["@mozilla.org/file/local;1"]
                    .createInstance(Components.interfaces.nsILocalFile);
  try
  {
    file.initWithPath(saveto);
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("cantSaveToPath");
    return;
  }
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                    .createInstance(Components.interfaces.nsIFileOutputStream);
  var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
                    .createInstance(Components.interfaces.nsIDOMSerializer);
  stream.init(
              file,
              /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */
              42,
              /*0644*/420,
              0
             );
  var savedom = model.dom;
  serializer.serializeToStream(savedom, stream, "UTF8");
}

function ChangeIntegratorButtons(d, intg, run)
{
  var expert = 0;
  try
  {
    var prefservice =
      Components.classes["@mozilla.org/preferences-service;1"]
               .getService(Components.interfaces.nsIPrefService);
    var cellBranch = prefservice.getBranch("physiome.mozcellml.");
    expert = cellBranch.getIntPref("expertlevel");
  }
  catch (e) {}

  SetVisibility(d.getElementById("compile-integrator"), !intg);
  SetVisibility(d.getElementById("kill-integrator"), intg && (expert > 1));
  SetVisibility(d.getElementById("start-integration"), (!run) && intg);
  SetVisibility(d.getElementById("cancel-integration"), run && intg);
}

function CompileIntegrator()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  // Get a temporary file name, to store the C code in...
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIDirectoryServiceProvider);
  var p = new Object();
  var lf = dirService.getFile("TmpD", p)
                     .QueryInterface(Components.interfaces.nsILocalFile);
  lf.appendRelativePath("integrator.c");
  lf.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0600*/384);
  var cfile = lf.path;
  var lfx = dirService.getFile("TmpD", p)
                      .QueryInterface(Components.interfaces.nsILocalFile);
  if (navigator.platform.substring(0, 5) == "Win32")
    lfx.appendRelativePath("integrator.exe");
  else
    lfx.appendRelativePath("integrator");

  lfx.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0700*/448);
  var xfile = lfx.path;

  var cc = GetCompiler();

  // XXX not all compilers take the same arguments...
  cc += " " + cfile + " -o " + xfile;

  var statusBar = document.getElementById('status-bar-display');

  statusBar.label = strb.getString("compilingIntegrator");
  ChangeIntegratorButtons(document, true, false);
  try
  {
    window.integrator = window.model.newIntegrator(cfile, cc, xfile);
    statusBar.label = strb.getString("integratorCompiled");;
  }
  catch (e)
  {
    var valerr = window.model.checkValidity();
    if (valerr == "")
      statusBar.label = strb.getString("compilationFailed");
    else
    {
      statusBar.label = strb.getFormattedString("integrationFailedReason",
                                                [valerr]);
    }
    ChangeIntegratorButtons(document, false, false);
  }
  // lf.remove(false);
}

function CleanupWindow()
{
  if (window.integrator != null)
  {
    window.integrator.destroy();
    window.integrator = null;
  }
}

function KillIntegrator()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  statusBar.label = strb.getString("killingIntegrator");
  window.integrator.destroy();
  window.integrator = null;
  ChangeIntegratorButtons(document, false, false);
  statusBar.label = strb.getString("integratorKilled");;
}

function IntegrationResultWatcher(npoints)
{
  this.data = new Array();
  this.curpoint = 0;
  this.npoints = npoints + 1;
  return this;
}

IntegrationResultWatcher.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozCellMLIPointWatcher) ||
        aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference)
       )
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  pointAdded: function(x, numvariables, variables, values)
  {
    var i, j;
    if (this.varmap == null)
    {
      this.varmap = new Array();
      for (i = 0; i < numvariables; i++)
      {
        for (j = 0; j < this.nvariables; j++)
          if (this.variables[j].equals(variables[i]))
          {
            this.varmap[i] = j;
            break;
          }
      }
    }
    this.data[this.curpoint] = new Array();
    for (i = 0; i < numvariables; i++)
      this.data[this.curpoint][this.varmap[i]] = values[i];
    this.curpoint++;
    if ((this.curpoint & 0xF) == 0)
    {
      if (this.meter != null)
        this.meter.setAttribute("value",
                                ((this.curpoint / this.npoints) * 100));
    }
    if (this.gc != null)
      this.gc.addDataPoint(numvariables, this.data[this.curpoint - 1]);
  },

  done: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationDone");
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
    this.curpoint = this.npoints;
    this.live = false;
    if (this.meter != null)
      this.meter.setAttribute("value", "100%");
  },

  halted: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("integrationHalted");
    this.live = false;
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
  },

  failed: function(emsg)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    var err = strb.getFormattedString("integrationFailedReason", [emsg]);
    statusBar.label = err;
    alert(err);
    this.live = false;
    if (window.integrator != null)
      ChangeIntegratorButtons(document, true, false);
  },

  live: false,
  closed: false
};

function StartIntegration()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  var startx = parseFloat(document.getElementById('startx').value);
  var stopx = parseFloat(document.getElementById('stopx').value);
  var gridx = parseFloat(document.getElementById('gridx').value);
  var relerr = parseFloat(document.getElementById('relerr').value);
  var abserr = parseFloat(document.getElementById('relerr').value);
  var maxstep = parseFloat(document.getElementById('maxstep').value);
  if (isNaN(startx) || !isFinite(startx))
  {
    statusBar.label = strb.getString("invalidStartingBound");
    return;
  }
  if (isNaN(stopx) || !isFinite(stopx))
  {
    statusBar.label = strb.getString("invalidEndingBound");
    return;
  }
  if (isNaN(gridx) || !isFinite(gridx))
  {
    statusBar.label = strb.getString("invalidBoundResolution");
    return;
  }
  if (isNaN(relerr) || !isFinite(relerr))
  {
    statusBar.label = strb.getString("invalidRelErr");
    return;
  }
  if (isNaN(abserr) || !isFinite(abserr))
  {
    statusBar.label = strb.getString("invalidAbsErr");
    return;
  }
  if (isNaN(maxstep) || !isFinite(maxstep))
  {
    statusBar.label = strb.getString("invalidMaxStep");
    return;
  }
  var diff = stopx - startx;
  if (diff < 0)
    diff = -diff;
  var irw = new IntegrationResultWatcher(parseInt(diff / gridx));
  irw.startx = startx;
  irw.stopx = stopx;
  irw.gridx = gridx;
  irw.model = window.model;
  irw.variables = new Array();
  var i = 0, j = 0;
  for (i = 0; i < window.nvariables; i++)
  {
    if (window.variables[i].type !=
        Components.interfaces.mozCellMLIVariable.BOUND &&
        window.variables[i].type !=
        Components.interfaces.mozCellMLIVariable.CONSTANT)
      irw.variables[j++] = window.variables[i];
  }
  irw.nvariables = j;
  irw.integrator = window.integrator;
  irw.maindoc = document;
  irw.mainwin = window;
  irw.bound = window.bound;
  irw.boundunit = window.boundunit;
  window.integrator.registerWatcher(irw);
  window.openDialog(
                    "chrome://mozcellml/content/integrationresult.xul",
                    "_blank",
                    "chrome,dialog=no",
                    irw
                   );
  irw.live = true;
  if (!irw.closed)
  {
    window.integrator.start(window.model, startx, stopx, gridx,
                            abserr, relerr, maxstep);
    ChangeIntegratorButtons(document, true, true);
    statusBar.label = strb.getString("integrationStarted");
  }
}

function CancelIntegrationWindow()
{
  window.arguments[0].closed = true;
  if (!window.arguments[0].live)
    return;
  if (window.arguments[0].mainwin.integrator != null)
  {
    ChangeIntegratorButtons(window.arguments[0].maindoc, true, false);
    window.arguments[0].mainwin.integrator.suspend();
  }
}

function CancelIntegration()
{
  if (window.integrator != null)
  {
    ChangeIntegratorButtons(document, true, false);
    window.integrator.suspend();
  }
}

function nyble2hex(x)
{
  if (x < 10)
    return "" + x;
  switch (x)
  {
    case 10:
      return 'A';
    case 11:
      return 'B';
    case 12:
      return 'C';
    case 13:
      return 'D';
    case 14:
      return 'E';
    case 15:
      return 'F';
  }
}

function hex2nyble(x)
{
  var c = x.charCodeAt(0);
  if (c >= 0x30 && c <= 0x39)
    return c - 0x30;
  if (c >= 0x41 && c <= 0x46)
    return c - 0x37;
  return c - 0x57;
}

function ascolour(val)
{
  return "#" + nyble2hex((val & 0xf0)>>4) +
               nyble2hex(val & 0xf) +
               nyble2hex((val & 0xf000)>>12) +
               nyble2hex((val & 0xf00)>>8) +
               nyble2hex((val & 0xf00000)>>20) +
               nyble2hex((val & 0xf0000)>>16);
}

function colourtonumber(col)
{
  // The hex number is written back-to-front...
  return (hex2nyble(col.substring(5, 6)) << 20) |
         (hex2nyble(col.substring(6, 7)) << 16) |
         (hex2nyble(col.substring(3, 4)) << 12) |
         (hex2nyble(col.substring(4, 5)) << 8) |
         (hex2nyble(col.substring(1, 2)) << 4) |
         hex2nyble(col.substring(2, 3));
}

function InsertIntegrationVariable(id, box, name, type, colour)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var li = document.createElement("listitem");
  li.setAttribute("id", name);
  var lc = document.createElement("listcell");
  lc.setAttribute("label", name);
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("label", type);
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("label", strb.getString("yes"));
  li.appendChild(lc);
  lc = document.createElement("listcell");
  lc.setAttribute("style", "background-color: " + ascolour(colour));
  li.appendChild(lc);
  box.appendChild(li);
}

function PrepareIntegrationWindow()
{
  var boundname = document.getElementById("boundname");
  boundname.value = window.arguments[0].bound;
  var boundunit = document.getElementById("boundunit");
  boundunit.value = window.arguments[0].boundunit;

  window.arguments[0].meter = document.getElementById("integration-progress");
  window.arguments[0].meter.setAttribute("value",
      ((window.arguments[0].curpoint / window.arguments[0].npoints) * 100));

  var box = document.getElementById("variable-tree");
  window.model = window.arguments[0].model;
  window.variables = window.arguments[0].variables;
  window.nvariables = window.arguments[0].nvariables;
  window.show = new Array();

  var slider = document.getElementById("xpos-scroll");
  var xbox = document.getElementById("xpos");
  slider.setAttribute("maxpos",
                      (window.arguments[0].stopx - window.arguments[0].startx) / 
                      window.arguments[0].gridx);
  xbox.value = window.arguments[0].startx;
  var i = 0;
  var nv = window.nvariables;
  var col = new Array();

  for (i = 0; i < nv; i++)
  {
    window.show[i] = true;
    var v = window.variables[i];
    col[i] = parseInt(Math.random() * 0xffffff);
    InsertIntegrationVariable("var" + i, box, v.name, TypeToString(v.type), col[i]);
  }
  window.colours = col;

  window.arguments[0].intwindow = window;

  var graphspace = document.getElementById("intg-graphspace");

  if (navigator.vendor == "Firefox")
    return;

  window.gc = Components.classes["@physiome.org/mozCellMLGraphClient;1"]
                        .createInstance(Components.interfaces.mozCellMLIGraphClient);

  window.gc.init(
                 window.arguments[0].startx,
                 window.arguments[0].stopx,
                 nv,
                 window.colours,
                 window.arguments[0].gridx,
                 graphspace
                );
  // XXX is there a race against pointAdded between these statements?
  var cp = window.arguments[0].curpoint;
  window.arguments[0].gc = window.gc;
  
  var i;
  for (i = 0; i < cp; i++)
    window.gc.addDataPoint(nv, window.arguments[0].data[i]);

  window.setTimeout("RedrawGraphspace()", 500);
}

function RedrawGraphspace()
{
  window.gc.paint();
  window.setTimeout("RedrawGraphspace()", 500);
}

function GetXAtTime(ti)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById("status-bar-display");
  if (ti < 0)
  {
    statusBar.label = strb.getString("beforeFirstPoint");
    document.getElementById("value-at-x").value = "";
    return;
  }
  if (ti >= window.arguments[0].curpoint)
  {
    statusBar.label = strb.getString("noDataAtPoint");
    document.getElementById("value-at-x").value = "";
    return;
  }
  var data = window.arguments[0].data[ti];
  if (data == null)
  {
    statusBar.label = strb.getString("noDataAtPoint");
    document.getElementById("value-at-x").value = "";
    return;
  }
  var box = document.getElementById("variable-tree");
  var idx = box.selectedIndex;
  if (idx == -1)
  {
    statusBar.label = strb.getString("selectVariableFirst");
    document.getElementById("value-at-x").value = "";
    return;
  }
  document.getElementById("value-at-x").value = data[idx];
  statusBar.label = strb.getString("valueRetrieved");
}

function SelectIntegrationVariable()
{
  var box = document.getElementById("variable-tree");
  var idx = box.selectedIndex;
  if (idx < 0)
    return;
  var name = window.variables[idx].name;
  document.getElementById("variable-name").value = name;
  document.getElementById("variable-unit").value = window.variables[idx].units;
  document.getElementById("draw-on-graph").checked = window.show[idx];
  var col = ascolour(window.colours[idx]);
  document.getElementById("graph-colour").color = col;
  GetXAtTime(document.getElementById("xpos-scroll").getAttribute("curpos"));
  return true;
}

function fitmodel()
{
  window.openDialog(
                    "chrome://mozcellml/content/fitmodel.xul",
                    "_blank",
                    "chrome,dialog=no",
                    window.model,
                    window.bound,
                    window.boundunit,
                    window.variables,
                    window.nvariables
                   );
}

function ChangeGraphColour()
{
  var box = document.getElementById("variable-tree");
  var idx = box.selectedIndex;
  if (idx < 0)
    return;
  var colour = document.getElementById("graph-colour").color;
  window.colours[idx] = colourtonumber(colour);
  box.selectedItem.childNodes.item(3)
     .setAttribute("style", "background-color: " + colour);
  if (navigator.vendor == "Firefox")
    return true;
  window.gc.setColour(idx, window.colours[idx]);
  return true;
}

function ChangeDraw()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var box = document.getElementById("variable-tree");
  var idx = box.selectedIndex;
  if (idx < 0)
    return;
  var draw = document.getElementById("draw-on-graph").checked;
  window.show[idx] = draw;
  box.selectedItem.childNodes.item(2)
     .setAttribute("label", draw ? strb.getString("yes") :
     strb.getString("no"));
  if (navigator.vendor == "Firefox")
    return false;
  window.gc.setVisibility(idx, draw);
  return false;
}

function ShowAll()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var i = 0, j = 0, l;
  var box = document.getElementById("variable-tree");
  document.getElementById("draw-on-graph").checked = true;
  var childvars = box.childNodes;
  l = childvars.length;
  for (i = 0; i < l; i++)
  {
    var cv = childvars.item(i);
    if (cv.localName != "listitem")
      continue;
    cv.childNodes.item(2).setAttribute("label", strb.getString("yes"));
    if (navigator.vendor != "Firefox")
      window.gc.setVisibility(j, true);
    window.show[j] = true;
    j++;
  }
}

function HideAll()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var i = 0, j = 0, l;
  var box = document.getElementById("variable-tree");
  document.getElementById("draw-on-graph").checked = false;
  var childvars = box.childNodes;
  l = childvars.length;
  for (i = 0; i < l; i++)
  {
    var cv = childvars.item(i);
    if (cv.localName != "listitem")
      continue;
    cv.childNodes.item(2).setAttribute("label", strb.getString("no"));
    if (navigator.vendor != "Firefox")
      window.gc.setVisibility(j, false);
    window.show[j] = false;
    j++;
  }
}

function OnXSlide()
{
  var slider = document.getElementById("xpos-scroll");
  var box = document.getElementById("xpos");
  box.value = slider.getAttribute("curpos") *
              window.arguments[0].gridx +
              window.arguments[0].startx;
  GetXAtTime(slider.getAttribute("curpos"));
  return true;
}

function OnXTextChange()
{
  var slider = document.getElementById("xpos-scroll");
  var box = document.getElementById("xpos");
  slider.setAttribute("curpos",
                      Math.floor((box.value - window.arguments[0].startx) /
                                 window.arguments[0].gridx));
  GetXAtTime(slider.getAttribute("curpos"));
  return true;
}

function CloneAtTime()
{
  var ti = document.getElementById("xpos-scroll").getAttribute("curpos");
  var statusBar = document.getElementById("status-bar-display");
  var strb = document.getElementById("mozcellml-string-bundle");
  if (ti < 0)
  {
    statusBar.label = strb.getString("beforeFirstPoint");
    return;
  }
  if (ti >= window.arguments[0].curpoint)
  {
    statusBar.label = strb.getString("noDataAtPoint");
    return;
  }
  var data = window.arguments[0].data[ti];
  var olddom = window.model.dom;
  var newdom = olddom.cloneNode(true);
  newdom =  newdom.QueryInterface(Components.interfaces.nsIDOMDocument);
  statusBar.label = strb.getString("copyingModel");
  var newmodel = Components.classes["@physiome.org/mozCellML;1"]
                  .createInstance(Components.interfaces.mozICellML);
  newmodel.setStringBundle(strb.stringBundle);
  newmodel.dom = newdom;
  newmodel.initialise();
  var i;
  statusBar.label = strb.getString("copyingVariables");
  for (i = 0; i < window.nvariables; i++)
  {
    try
    {
      // Throw an exception if no initial value...
      var x = window.variables[i].initialValue;
      newmodel.getVariable(window.variables[i].name,
                           window.variables[i].component).initialValue =
        data[i];
    }
    catch (e)
    {
      // If there was no initial value before, we don't want one now.
    }
  }
  statusBar.label = strb.getString("creatingWindow");
  window.arguments[0].integrator.want();
  window.openDialog(
                    "chrome://mozcellml/content/modelview.xul",
                    "_blank",
                    "chrome,dialog=no",
                    newmodel,
                    window.arguments[0].integrator
                   );
  statusBar.label = strb.getString("modelCloned");
}

function StreamPrint(strm, strng)
{
  strng += "\r\n";
  strm.write(strng, strng.length);
}

function GetVariableDescriptor(v, i)
{
  var vd = v[i].name;
  try
  {
    vd += "(" + v[i].units + ")";
  }
  catch (e)
  {
  }
  return vd;
}

function SaveCSV()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var filename = document.getElementById('csvfile').value;
  var file = Components.classes["@mozilla.org/file/local;1"]
                       .createInstance(Components.interfaces.nsILocalFile);
  try
  {
    file.initWithPath(filename);
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("cantSaveToPath");
    return;
  }
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                    .createInstance(Components.interfaces.nsIFileOutputStream);
  stream.init(
              file,
              /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */
              42,
              /*0644*/420,
              0
             );
  var intg = window.arguments[0];
  var names = "";
  if (intg.nvariables == 0)
  {
    stream.close();
    return;
  }
  names += intg.bound + "(" + intg.boundunit + ")";
  for (i = 0; i < intg.nvariables; i++)
    names += "," + GetVariableDescriptor(intg.variables, i);
  StreamPrint(stream, names);
  var boundval = intg.startx;
  for (i = 0; i < intg.curpoint; i++)
  {
    var values = boundval;
    boundval += intg.gridx;
    for (j = 0; j < intg.nvariables; j++)
      values += "," + intg.data[i][j];
    StreamPrint(stream, values);
  }
  stream.close();
  var statusBar = document.getElementById('status-bar-display');
  statusBar.label = strb.getString("csvExportComplete");
}
